searchData={"items":[{"type":"module","title":"Commandex","doc":"Defines a command struct.\n\nCommandex is a loose implementation of the command pattern, making it easy\nto wrap parameters, data, and errors into a well-defined struct.","ref":"Commandex.html"},{"type":"module","title":"Example - Commandex","doc":"A fully implemented command module might look like this:\n\n    defmodule RegisterUser do\n      import Commandex\n\n      command do\n        param :email\n        param :password\n\n        data :password_hash\n        data :user\n\n        pipeline :hash_password\n        pipeline :create_user\n        pipeline :send_welcome_email\n      end\n\n      def hash_password(command, %{password: nil} = _params, _data) do\n        command\n        |> put_error(:password, :not_given)\n        |> halt()\n      end\n\n      def hash_password(command, %{password: password} = _params, _data) do\n        put_data(command, :password_hash, Base.encode64(password))\n      end\n\n      def create_user(command, %{email: email} = _params, %{password_hash: phash} = _data) do\n        %User{}\n        |> User.changeset(%{email: email, password_hash: phash})\n        |> Repo.insert()\n        |> case do\n          {:ok, user} -> put_data(command, :user, user)\n          {:error, changeset} -> command |> put_error(:repo, changeset) |> halt()\n        end\n      end\n\n      def send_welcome_email(command, _params, %{user: user}) do\n        Mailer.send_welcome_email(user)\n        command\n      end\n    end\n\nThe `command/1` macro will define a struct that looks like:\n\n    %RegisterUser{\n      success: false,\n      halted: false,\n      errors: %{},\n      params: %{email: nil, password: nil},\n      data: %{password_hash: nil, user: nil},\n      pipelines: [:hash_password, :create_user, :send_welcome_email]\n    }\n\nAs well as two functions:\n\n    &RegisterUser.new/1\n    &RegisterUser.run/1\n\n`&new/1` parses parameters into a new struct. These can be either a keyword list\nor map with atom/string keys.\n\n`&run/1` takes a command struct and runs it through the pipeline functions defined\nin the command. **Functions are executed in the order in which they are defined**.\nIf a command passes through all pipelines without calling `halt/1`, `:success` \nwill be set to `true`. Otherwise, subsequent pipelines after the `halt/1` will \nbe ignored and `:success` will be set to `false`.\n\n    %{email: \"example@example.com\", password: \"asdf1234\"}\n    |> RegisterUser.new()\n    |> RegisterUser.run()\n    |> case do\n      %{success: true, data: %{user: user}} ->\n        # Success! We've got a user now\n\n      %{success: false, errors: %{password: :not_given}} ->\n        # Respond with a 400 or something\n\n      %{success: false, errors: _error} ->\n        # I'm a lazy programmer that writes catch-all error handling\n    end","ref":"Commandex.html#module-example"},{"type":"module","title":"Parameter-less Commands - Commandex","doc":"If a command does not have any parameters defined, a `run/0` will be generated\nautomatically. Useful for diagnostic jobs and internal tasks.\n\n    iex> GenerateReport.run()\n    %GenerateReport{\n      pipelines: [:fetch_data, :calculate_results],\n      data: %{total_valid: 183220, total_invalid: 781215},\n      params: %{},\n      halted: false,\n      errors: %{},\n      success: true\n    }","ref":"Commandex.html#module-parameter-less-commands"},{"type":"macro","title":"Commandex.command/1","doc":"Defines a command struct with params, data, and pipelines.","ref":"Commandex.html#command/1"},{"type":"macro","title":"Commandex.data/1","doc":"Defines a command data field.\n\nData field values are created and set as pipelines are run. Set one with `put_data/3`.\n\n    command do\n      # ...params\n\n      data :password_hash\n      data :user\n\n      # ...pipelines\n    end","ref":"Commandex.html#data/1"},{"type":"function","title":"Commandex.halt/1","doc":"Halts a command pipeline.\n\nAny pipelines defined after the halt will be ignored. If a command finishes running through\nall pipelines, `:success` will be set to `true`.\n\n    def hash_password(command, %{password: nil} = _params, _data) do\n      command\n      |> put_error(:password, :not_supplied)\n      |> halt()\n    end","ref":"Commandex.html#halt/1"},{"type":"macro","title":"Commandex.param/2","doc":"Defines a command parameter field.\n\nParameters are supplied at struct creation, before any pipelines are run.\n\n    command do\n      param :email\n      param :password\n\n      # ...data\n      # ...pipelines\n    end","ref":"Commandex.html#param/2"},{"type":"macro","title":"Commandex.pipeline/1","doc":"Defines a command pipeline.\n\nPipelines are functions executed against the command, *in the order in which they are defined*.\n\nFor example, two pipelines could be defined:\n  \n    pipeline :check_valid_email\n    pipeline :create_user\n\nWhich could be mentally interpreted as:\n\n    command\n    |> check_valid_email()\n    |> create_user()\n\nA pipeline function can be defined multiple ways:\n\n- `pipeline :do_work` - Name of a function inside the command's module, arity three.\n- `pipeline {YourModule, :do_work}` - Arity three.\n- `pipeline {YourModule, :do_work, [:additonal, \"args\"]}` - Arity three plus the \n  number of additional args given.\n- `pipeline &YourModule.do_work/1` - Or any anonymous function of arity one.\n- `pipeline &YourModule.do_work/3` - Or any anonymous function of arity three.","ref":"Commandex.html#pipeline/1"},{"type":"function","title":"Commandex.put_data/3","doc":"Sets a data field with given value.\n\nDefine a data field first:\n\n    data :password_hash\n\nSet the password pash in one of your pipeline functions:\n\n    def hash_password(command, %{password: password} = _params, _data) do\n      # Better than plaintext, I guess\n      put_data(command, :password_hash, Base.encode64(password))\n    end","ref":"Commandex.html#put_data/3"},{"type":"function","title":"Commandex.put_error/3","doc":"Sets error for given key and value.\n\n`:errors` is a map. Putting an error on the same key will overwrite the previous value.\n\n    def hash_password(command, %{password: nil} = _params, _data) do\n      command\n      |> put_error(:password, :not_supplied)\n      |> halt()\n    end","ref":"Commandex.html#put_error/3"},{"type":"type","title":"Commandex.command/0","doc":"Command struct.","ref":"Commandex.html#t:command/0"},{"type":"type","title":"Attributes - Commandex.command/0","doc":"- `data` - Data generated during the pipeline, defined by `Commandex.data/1`.\n- `errors` - Errors generated during the pipeline with `Commandex.put_error/3`\n- `halted` - Whether or not the pipeline was halted.\n- `params` - Parameters given to the command, defined by `Commandex.param/1`.\n- `pipelines` - A list of pipeline functions to execute, defined by `Commandex.pipeline/1`.\n- `success` - Whether or not the command was successful. This is only set to\n  `true` if the command was not halted after running all of the pipelines.","ref":"Commandex.html#t:command/0-attributes"},{"type":"type","title":"Commandex.pipeline/0","doc":"Command pipeline stage.\n\nA pipeline function can be defined multiple ways:\n\n- `pipeline :do_work` - Name of a function inside the command's module, arity three.\n- `pipeline {YourModule, :do_work}` - Arity three.\n- `pipeline {YourModule, :do_work, [:additonal, \"args\"]}` - Arity three plus the \n  number of additional args given.\n- `pipeline &YourModule.do_work/1` - Or any anonymous function of arity one.\n- `pipeline &YourModule.do_work/3` - Or any anonymous function of arity three.","ref":"Commandex.html#t:pipeline/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.5.1] - 2024-09-09","ref":"changelog.html"},{"type":"extras","title":"Fixed - Changelog","doc":"- `import_deps: [:commandex]` in `.formatter.exs` works again. ([#15](https://github.com/codedge-llc/commandex/pull/15))\n\n## [0.5.0] - 2024-09-08","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- `run/0` function for commands that don't define any parameters.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Raise `ArgumentError` if an invalid `pipeline` is defined.\n\n## [0.4.1] - 2020-06-26","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Set `false` parameter correctly when given a Map of params. Was previously\n  evaluating to `nil`.\n\n## [0.4.0] - 2020-05-03","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- Default typespecs and documentation for modules using Commandex.\n  Note: this will break any existing modules that have `@type t` already defined.\n\n## [0.3.0] - 2020-01-31","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- `param` now supports a `:default` option. (eg. `param :limit, default: 10`)\n- Added `new/0` to initialize commands without any parameters.\n- `pipeline` can now use a 1-arity anonymous function. (eg. `pipeline &IO.inspect/1`)\n\n## [0.2.0] - 2020-01-21","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Enhanced documentation to show `&run/1` shortcut","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Renamed `:error` to `:errors` on Command struct\n\n## [0.1.0] - 2020-01-18\n\n- Initial release","ref":"changelog.html#changed"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2020-2024 Codedge LLC (https://www.codedge.io/)\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}